# application that shows normal distribution superimposed to a binomial distribution
# author: Sergio Miranda Freire

library(shiny)
library(RcmdrMisc)
resolution <- 0.1

server <- function(input, output) {
    n <- reactive ({
        input$n
    })
    p <- reactive ({
        input$p
    })
    mean <- reactive ({
        n() * p()
    })
    sd_ <- reactive ({
        sqrt(n() * p() * (1 - p()))
    })
    
    output$binomial_normal <- renderPlot({
        .x <- 0:n()
        plotDistr(
            .x,
            dbinom(.x, size = n(), prob = p()),
            ylim = c(0, dnorm(mean(), mean(), sd_())),
            xlab = "Numero de Sucessos",
            ylab = "Distribuição de Probabilidades",
            main = paste("Distribuicao Binomial:  np = ", sprintf("%.1f", mean()), "    n x (1-p) = ", sprintf("%.1f", n() * (1-p())), sep=""),
            discrete = TRUE
        )
        curve(
            dnorm(x, mean(), sd_()),
            xlim = c(0, n()),
            main = "",
            xlab = "",
            ylab = "",
            yaxt = 'n',
            ylim = c(0, dnorm(mean(), mean(), sd_())),
            add = TRUE,
            col = "blue"
        )
        cord.x <- c(5,seq(5,8,resolution),8) 
        cord.y <- c(0,dnorm(seq(5,8,resolution), mean(), sd_()), 0)
#        polygon(cord.x,cord.y,col=rgb(1,1, 0,0.5))
        cord.x <- c(4.5,seq(4.5,8.5,resolution),8.5) 
        cord.y <- c(0,dnorm(seq(4.5,8.5,resolution), mean(), sd_()), 0)
        polygon(cord.x, cord.y, col=rgb(1,0, 0,0.5))
        
    })
}


# Define UI for application that draws a roc curve
ui <- fluidPage(# Application title
    titlePanel("Binomial x Normal"),
    
    sidebarLayout(sidebarPanel(
        sliderInput(
            "n",
            label = "Numero de experimentos",
            min = 5,
            max = 1000,
            value = 12
        ),
        sliderInput(
            "p",
            label = "probabilidade de sucesso",
            min = 0.0001,
            max = 1,
            value = 0.5
        )
),
    
    # Show plots
    mainPanel(plotOutput("binomial_normal")))
)

shinyApp(ui = ui, server = server)