#
# applicação que mostra a comparação de dois testes de diagnóstico
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
resolution <- 0.1

server <- function(input, output) {

  # ponto de corte para o cálculo da especificidade e sensibilidade do teste 1
  cutoff1 <- reactive ({
    input$cutoff1
  })

  # média da distribuição dos não doentes no teste 1  
  mean_inf1 <- reactive ({
    input$meanNormal1
  })
  
  # desvio padrão da distribuição dos não doentes no teste 1
  sd_inf1 <- reactive ({
    input$dpNormal1
  })
  
  # média da distribuição dos doentes no teste 1  
  mean_sup1 <- reactive ({
    input$meanDoente1
  })
  
  # desvio padrão da distribuição dos doentes no teste 1
  sd_sup1 <- reactive ({
    input$dpDoente1
  })
  
  # limite inferior para a construção dos gráficos para o teste 1
  inf_limit1 <- reactive ({
      mean_inf1() - 4 * sd_inf1()
  })

  # limite superior para a construção dos gráficos para o teste 2
  sup_limit1 <- reactive ({
    mean_sup1() + 4 * sd_sup1()
  })
  
  # ponto de corte para o cálculo da especificidade e sensibilidade do teste 2
  cutoff2 <- reactive ({
      input$cutoff2
  })
  
  # média da distribuição dos não doentes no teste 2  
  mean_inf2 <- reactive ({
      input$meanNormal2
  })
  
  # desvio padrão da distribuição dos não doentes no teste 2
  sd_inf2 <- reactive ({
      input$dpNormal2
  })
  
  # média da distribuição dos doentes no teste 2  
  mean_sup2 <- reactive ({
      input$meanDoente2
  })
  
  # desvio padrão da distribuição dos doentes no teste 2
  sd_sup2 <- reactive ({
      input$dpDoente2
  })
  
  # limite inferior para a construção dos gráficos para o teste 2
  inf_limit2 <- reactive ({
      mean_inf2() - 4 * sd_inf2()
  })
  
  # limite superior para a construção dos gráficos para o teste 2
  sup_limit2 <- reactive ({
      mean_sup2() + 4 * sd_sup2()
  })  
  
  # menor valor entre os dois limites inferiores para a construção dos gráficos
  inf_limit <- reactive ({
      min(inf_limit1(), inf_limit2())
  })
  
  # maior valor entre os dois limites superiores para a construção dos gráficos
  sup_limit <- reactive ({
      max(sup_limit1(), sup_limit2())
  })
  
  output$test1 <- renderPlot({
    adjX <- (mean_inf1() - inf_limit()) / (sup_limit() - inf_limit())
    # coordenadas para o desenho da área correspondendo aos falsos positivos do teste 1
    cord.x <- c(cutoff1(),seq(cutoff1(),sup_limit1(),resolution),sup_limit1()) 
    cord.y <- c(0,dnorm(seq(cutoff1(),sup_limit1(),resolution), mean_inf1(), sd_inf1()),0)
    # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
    ymax = max(dnorm(mean_inf1(),mean_inf1(),sd_inf1()), dnorm(mean_sup1(),mean_sup1(),sd_sup1()))
    curve(dnorm(x,mean_inf1(),sd_inf1()),xlim=c(inf_limit(),sup_limit()), main=paste("N(",sprintf("%.1f", mean_inf1()),", ",sprintf("%.1f", sd_inf1()*sd_inf1()),")", sep=""), adj=adjX, xlab = "", col= "blue", ylab = "", xaxt = 'n', yaxt='n', ylim = c(0, ymax), lwd = 2) 
    # desenho da área correspondente aos falsos positivos do teste 1
    polygon(cord.x,cord.y,col=rgb(1,1, 0,0.5))
    par(new=TRUE)
    adjX <- (mean_sup1() - inf_limit()) / (sup_limit() - inf_limit())
    # gráfico da densidade de probabilidade dos doentes no teste 1
    curve(dnorm(x,mean_sup1(),sd_sup1()),xlim=c(inf_limit(),sup_limit()),main="", col= "blue", xlab = "", ylab = "", yaxt='n', ylim = c(0, ymax), lwd = 2)
     par(cex=1.5)
    mtext(paste("N(",sprintf("%.1f", mean_sup1()),", ",sprintf("%.1f", sd_sup1()*sd_sup1()),")", sep=""), adj= adjX, cex = 1.2, font = 2)
    title(xlab="Teste 1", adj = 0.4)
    # coordenadas para o desenho da área correspondendo aos falsos negativos do teste1
    cord.z <- c(inf_limit1(),seq(inf_limit1(),cutoff1(),resolution),cutoff1())
    cord.w <- c(0,dnorm(seq(inf_limit1(),cutoff1(),resolution), mean_sup1(), sd_sup1()),0)
    # desenho da área correspondente aos falsos negativos do teste 1
    polygon(cord.z,cord.w,col=rgb(0, 1, 0,0.5))
    limits <- par("usr")
    ypos <- 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
    xpos <- mean_sup1() + 1.5*(sup_limit1() - mean_sup1())/5
    par(cex=1.1)
    legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","green"))
  })

  output$test2 <- renderPlot({
      adjX <- (mean_inf2() - inf_limit()) / (sup_limit() - inf_limit())
      # coordenadas para o desenho da área correspondendo aos falsos positivos do teste 2
      cord.x <- c(cutoff2(),seq(cutoff2(),sup_limit2(),resolution),sup_limit2()) 
      cord.y <- c(0,dnorm(seq(cutoff2(),sup_limit2(),resolution), mean_inf2(), sd_inf2()),0)
      # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
      ymax = max(dnorm(mean_inf2(),mean_inf2(),sd_inf2()), dnorm(mean_sup2(),mean_sup2(),sd_sup2()))
      # gráfico da densidade de probabilidade dos não doentes no teste 2
      curve(dnorm(x,mean_inf2(),sd_inf2()),xlim=c(inf_limit(),sup_limit()), main=paste("N(",sprintf("%.1f", mean_inf2()),", ",sprintf("%.1f", sd_inf2()*sd_inf2()),")", sep=""), adj=adjX, col = "red", xlab = "", ylab = "", xaxt = 'n', yaxt='n', ylim = c(0, ymax), lwd = 2) 
      # desenho da área correspondente aos falsos positivos do teste 2
      polygon(cord.x,cord.y,col=rgb(1,1, 0,0.5))
      par(new=TRUE)
      adjX <- (mean_sup2() - inf_limit()) / (sup_limit() - inf_limit())
      # gráfico da densidade de probabilidade dos doentes no teste 2
      curve(dnorm(x,mean_sup2(),sd_sup2()),xlim=c(inf_limit(),sup_limit()),main="", col= "red", xlab = "", ylab = "", yaxt='n', ylim = c(0, ymax), lwd = 2)
      par(cex=1.5)
      mtext(paste("N(",sprintf("%.1f", mean_sup2()),", ",sprintf("%.1f", sd_sup2()*sd_sup2()),")", sep=""), adj= adjX, cex = 1.2, font = 2)
      title(xlab="Teste 2", adj = 0.4)
      # coordenadas para o desenho da área correspondendo aos falsos negativos do teste 2
      cord.z <- c(inf_limit2(),seq(inf_limit2(),cutoff2(),resolution),cutoff2())
      cord.w <- c(0,dnorm(seq(inf_limit2(),cutoff2(),resolution), mean_sup2(), sd_sup2()),0)
      # desenho da área correspondente aos falsos negativos
      polygon(cord.z,cord.w,col=rgb(0, 1, 0,0.5))
      limits <- par("usr")
      ypos <- 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
      xpos <- mean_sup2() + 1.5*(sup_limit2() - mean_sup1())/5
      par(cex=1.1)
      legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","green"))
  })
  
  output$roc <- renderPlot({
    par(mar=c(5,5,1,2))
    curve(pnorm(qnorm(x, mean_inf1(), sd_inf1(), lower.tail=FALSE), mean_sup1(), sd_sup1(), lower.tail=FALSE) ,xlim=c(0,1),main="", xlab = "", ylab = "", col = "blue", lwd = 2)
    # especificidade e sensibilidade correspondentes ao ponto de corte do teste 1
    x <- pnorm(cutoff1(),mean_inf1(), sd_inf1(), lower.tail=FALSE)
    y <- pnorm(cutoff1(),mean_sup1(), sd_sup1(), lower.tail=FALSE)
    # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade do teste 1
    points(x, y, col = "black", pch=21, cex = 2.0, bg = "black")
    text(x +.01, y -.05, paste("S = ", sprintf("%.2f", y)), pos = 4)
    text(x +.01, y -.1, paste("E = ", sprintf("%.2f", 1-x)), pos = 4)
    curve(pnorm(qnorm(x, mean_inf2(), sd_inf2(), lower.tail=FALSE), mean_sup2(), sd_sup2(), lower.tail=FALSE) ,xlim=c(0,1),main="", xlab = "", ylab = "", col = "red", lwd = 2, add = T)
    # especificidade e sensibilidade correspondentes ao ponto de corte do teste 2
    x <- pnorm(cutoff2(),mean_inf2(), sd_inf2(), lower.tail=FALSE)
    y <- pnorm(cutoff2(),mean_sup2(), sd_sup2(), lower.tail=FALSE)
    # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade do teste 2
    points(x, y, col = "black", pch=21, cex = 2.0, bg = "black")
    text(x +.01, y -.05, paste("S = ", sprintf("%.2f", y)), pos = 4)
    text(x +.01, y -.1, paste("E = ", sprintf("%.2f", 1-x)), pos = 4)
    par(cex=1.5)
    title(xlab = "1 - Especificidade", ylab = "Sensibilidade")
    })
}


ui <- fluidPage(
  
  titlePanel("Curvas ROC para dois testes"),
  
  sidebarLayout(
    sidebarPanel(
        sliderInput(
            "meanNormal1",
            label = expression("Média da população Normal Teste 1"),
            min = 60,
            max = 110,
            value = 92
        ),
        sliderInput(
            "dpNormal1",
            label = expression("Desvio-padrão da população Normal Teste 1"),
            min = 6,
            max = 50,
            value = 16
        ),
        sliderInput(
            "meanDoente1",
            label = expression("Média da população Doente Teste 1"),
            min = 110,
            max = 180,
            value = 140
        ),
        sliderInput(
            "dpDoente1",
            label = expression("Desvio-padrão da população Doente Teste 1"),
            min = 6,
            max = 50,
            value = 23
        ),
        sliderInput(
            "cutoff1",
            "Ponto de corte Teste 1",
            min = 40,
            max = 210,
            value = 110
        ),
        sliderInput(
            "meanNormal2",
            label = expression("Média da população Normal teste 2"),
            min = 110,
            max = 160,
            value = 140
        ),
        sliderInput(
            "dpNormal2",
            label = expression("Desvio-padrão da população Normal Teste 2"),
            min = 6,
            max = 50,
            value = 10
        ),
        sliderInput(
            "meanDoente2",
            label = expression("Média da população Doente Teste 2"),
            min = 150,
            max = 200,
            value = 170
        ),
        sliderInput(
            "dpDoente2",
            label = expression("Desvio-padrão da população Doente Teste 2"),
            min = 6,
            max = 50,
            value = 10
        ),
        sliderInput(
            "cutoff2",
            "Ponto de corte Teste 2",
            min = 80,
            max = 230,
            value = 160
        )
    ),
    
    # Show plots
    mainPanel(fluidRow(
        column(8, plotOutput("test1")),
        column(8, plotOutput("test2")),
        column(8, plotOutput("roc"))
    ))
  )
)


shinyApp(ui = ui, server = server)