# applicacao que mostra a analise de variancia em um modelo de regressao linear
# autor: Sergio Miranda Freire
# a soma dos quadrados total somente é a soma SSR + SSE na reta de regressão 

library(shiny)
library(car)
set.seed(333)
data(Prestige, package="car")
x <- Prestige$education
n <- length(x)
y <- 10 + 5 * x + rnorm(n, 0, 5)

server <- function(input, output) {
 
  b <- reactive ({
    input$beta
  })

  a <- reactive ({
      mean(y) - b() * mean(x)
  })
  
  variance_table <- reactive ({
      SQE <- sum((y - (a() + b() * x))^2)
      QME <- SQE / (n - 2)
      SQT <- sum(y^2) - n * (mean(y))^2
      SQR<- sum(((a() + b() * x) - mean(y))^2)   
      QMR<- SQR
      F_ <- QMR / QME
      sqt <- paste(sprintf("%.2f", SQT))
      sqe <- paste(sprintf("%.2f", SQE))
      qme <- paste(sprintf("%.2f", QME))
      sqr <- paste(sprintf("%.2f", SQR))
      qmr <- paste(sprintf("%.2f", QMR))
      f <- paste(sprintf("%.2f", F_))
      variance_components_table <- matrix(nrow = 0, ncol = 5)
      gl1 <- paste("n - p = ", sprintf("%3d", n-2))
      gl2 <- paste("p - 1 = 1")
      gl3 <- paste("n - 1 = ", sprintf("%3d", n-1))
      component1 <- "Residual"
      component2 <- "Regressão"
      component3 <- "Total"
      text1 <- c(component1, gl1, sqe, qme, f)
      variance_components_table <- rbind(variance_components_table, text1)
      text2 <- c(component2, gl2, sqr, qmr, "")
      variance_components_table <- rbind(variance_components_table, text2)
      text3 <- c(component3, gl3, sqt, "", "")
      variance_components_table <- rbind(variance_components_table, text3)
      colnames(variance_components_table) <- c("Fonte de variação", "Graus de liberdade", "Soma dos quadrados", "Quadrado médio", "F")
      variance_components_table
  })
  
  output$mstot <- renderPlot({
      plot(x, y, main = "Soma dos quadrados total", xlab = "", ylab = "",  xlim = c(0, 20), ylim = c(0, 100))
      abline(h = mean(y), lwd = 2)
      segments(x, y, x, mean(y), col = "red")
  })

  output$mse <- renderPlot({
      plot(x, y, main = "Soma dos quadrados residual",  xlab = "", ylab = "", xlim = c(0, 20), ylim = c(0, 100))
      segments(x, y, x, (a() + b() * x), col = "red")
      abline(a(), b(), lwd = 2, col = "blue")
  })
  
  output$msr <- renderPlot({
      plot(x, y, main = "Soma dos quadrados devido à regressão", xlab = "", ylab = "", xlim = c(0, 20),  ylim = c(0, 100))
      segments(x, mean(y), x, (a() + b() * x), col = "red")
      abline(a(), b(), lwd = 2, col = "blue")
      abline(h=mean(y), lwd = 2)
  })
  
  output$variance_components <- renderTable({variance_table()}, align = 'r')
}


ui <- fluidPage(
  
  # Application title
  titlePanel(paste(expression(Regressão), " Linear")),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("beta", 
                  paste(expression(Inclinação), " da reta:"), 
                  min = 0, 
                  max = 10, 
                  step = .001,
                  value = 3
      )
    ),
      
    # Show plots
    mainPanel(
        fluidRow(
            column(5, plotOutput("mse")),
            column(5, plotOutput("msr"))
        ),
        fluidRow(
            column(5, plotOutput("mstot")),
            column(5, align="center", tableOutput("variance_components"))
       ))
  )
)



shinyApp(ui = ui, server = server)