#
# aplicação que mostra o conceito de integração
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)

# Distribuição normal que será utilizada N(80, 400)
popMean <- 80
popSd <- 20
minAxis <-  popMean - 1.5 * popSd
maxAxis <- popMean + 1.5 * popSd

server <- function(input, output) {

  # número de retângulos que serão usados para aproximar a integral
  n <- reactive ({
    input$n
  })
 
  # plota a função densidade de probabilidade e os retângulos que irão aproximar a área 
  # sob a função densidade de probabilidade
  output$integral <- renderPlot({
    # limites inferior e superior de integração
    a <- popMean - 1.2 * popSd
    b <- popMean + 1 * popSd
    # base de cada retângulo
    delta <- (b - a) / n()
    # gráfico da função densidade de probabilidade N(80, 400)
    curve(dnorm(x,popMean, popSd),xlim=c(minAxis, maxAxis), ylim=c(-0.0025, 0.022), main="Densidade de Probabilidade", xlab = "", ylab = "", axes = FALSE) 
    axis(1, pos=0, labels = FALSE)
    axis(2, pos=49)
    # traça os limites dos retângulos
    cord.x <- NULL
    cord.y <- 0
    for ( i in c(0:n())) {
      cord.x <- c(cord.x, rep(a + i * delta, 2))
      cord.y <- c(cord.y, rep(dnorm(a + i * delta, popMean, popSd), 2))
      segments(a + i * delta, 0, a + i * delta, dnorm(a + i * delta, popMean, popSd) )
    }
    # preenche as áreas dos retângulos
    k <- 2*n()+2
    cord.y <- c(cord.y[1:k],0)
    cord.x <- c(cord.x, b)
    polygon(cord.x,cord.y,col=rgb(1,0, 0,0.5))
    # escreve 
    par(cex=1.2)
    delta <- (cord.x[5] - cord.x[3]) * .1
    text(cord.x[1], cord.y[2] + 0.001,  "f(a)")
    text(cord.x[k-2]+2, cord.y[k-1] + 0.001,  labels=expression(paste("f(x"["n-1"], ") ")))
    # para menos de 10 retângulos, mostra a expressão do cálculo das áreas 
    if (n() < 10) {
        segments(cord.x[3] + delta, 0, cord.x[3] + delta,  cord.y[4], col= "blue", lwd = 2)
        segments(cord.x[3] + delta - .4, 0, cord.x[3] + delta + .4, 0, col = "blue", lwd = 2)
        segments(cord.x[3] + delta - .4, cord.y[4], cord.x[3]+ delta + .4, cord.y[4], col = "blue", lwd = 2)
        segments(cord.x[3], -0.00125, cord.x[5],  -0.00125, col= "blue", lwd = 2)
        segments(cord.x[3], -0.00075, cord.x[3], -0.00175, col = "blue", lwd = 2)
        segments(cord.x[5], -0.00075, cord.x[5], -0.00175, col = "blue", lwd = 2)
        text((cord.x[3] + cord.x[5]) / 2, -0.002, expression(paste(Delta, "x")))
        text(cord.x[3], cord.y[4] + .001,  labels=expression(paste("f(x"[1], ") ")))
        text(cord.x[3] + (-cord.x[3] + cord.x[5]) * 3/5, cord.y[4] /2,  labels=expression(paste("Área = f(x"[1], ") * ", Delta, "x")))
        text(cord.x[3], - 0.003,  labels=expression(paste("x"[1])))
    }
    text(a, - 0.0015,  labels=expression(paste("a = x" [0])))
    text(b, - 0.0015,  labels=expression(paste("b = x" [n])))
    if (n() == 200) { # se n = 200 substitui o símbolo de somatório pelo de integração
        text(a, 0.015,  labels=expression(paste("Área total = ", integral(f(x) * dx, a,b))))
    } else {
        text(a, 0.015,  labels=expression(paste("Área total = ", sum(f(x[i]), i==0, n-1), " * ", Delta, "x")))
    }
  })

  output$cumulativeDistribution <- renderPlot({
    # limites inferior e superior de integração
    a <- popMean - 1.2 * popSd
    b <- popMean + 1 * popSd
    # base de cada retângulo
    delta <- (b - a) / n()
    # gráfico da função distribuição acumulada de probabilidade N(80, 400)
    curve(pnorm(x,popMean, popSd, lower.tail = TRUE),xlim=c(minAxis, maxAxis), ylim=c(-0.2, 1.2), main="Distribuição Acumulada", axes = FALSE, xlab = "", ylab = "") 
    axis(1, pos=0, labels = FALSE)
    axis(2, pos=49)
    # calcula as coordenadas da função de probabilidade acumulada nos limites de cada retângulo
    cord.x <- c(a, a)
    Fx <- pnorm(a, popMean, popSd, lower.tail = TRUE)
    cord.y <- c(0, Fx, Fx)
    for ( i in c(1:n())) {
      cord.x <- c(cord.x, rep(a + i * delta, 2))
      Fx <- Fx + delta * dnorm(a + (i - 1) * delta, popMean, popSd)
      cord.y <- c(cord.y, rep(Fx, 2))
    }
    k <- 2*n()+2
    cord.y <- c(cord.y[1:k],0)
    cord.x <- c(cord.x, b)
    # mostra os incrementos no valor da probabilidade acumulada 
    polygon(cord.x,cord.y,col=rgb(1,1, 1,0.5))
    par(cex=1.2)
    # para menos de 10 retângulos, mostra o cálculo dos incrementos no valor da probabilidade acumulada 
    if (n() < 10) {
        segments(cord.x[5]+1, cord.y[4], cord.x[5]+1,  cord.y[6], col= "blue", lwd = 2)
        segments(cord.x[5]+.5, cord.y[4], cord.x[5]+1.5, cord.y[4], col = "blue", lwd = 2)
        segments(cord.x[5]+.5, cord.y[6], cord.x[5]+1.5, cord.y[6], col = "blue", lwd = 2)
        text(cord.x[5] + 6, (cord.y[6] + cord.y[4]) / 2, expression(paste(Delta, "F "," = ", "F(x + ", Delta, "x) - F(x)")))
        segments(cord.x[3], -0.1, cord.x[5],  -0.1, col= "blue", lwd = 2)
        segments(cord.x[3], -0.075, cord.x[3], -0.125, col = "blue", lwd = 2)
        segments(cord.x[5], -0.075, cord.x[5], -0.125, col = "blue", lwd = 2)
        text(cord.x[3], - 0.03,  labels=expression(paste("x"[1])))
        text((cord.x[5] + cord.x[3]) / 2, -0.2, expression(paste(Delta, "x")))
    }
    text(cord.x[1], cord.y[2] + 0.05,  "F(a)")
    text(b+1, cord.y[k] + 0.095,  labels=expression(paste("F(x" ["n-1"], ") + ", Delta, "F")))
    text(a, - 0.05, labels=expression(paste("a = x" [0])))
    text(b, - 0.05,  labels=expression(paste("b = x" [n])))
  })
  
  }

ui <- fluidPage(
  
  titlePanel("Integração"),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("n", label = expression("Número de pontos"), 
                min = 4, max = 200, step = 1, value = 4)
    ),
    
    mainPanel(
      plotOutput("integral"),
      plotOutput("cumulativeDistribution")
    )
  )
)

shinyApp(ui = ui, server = server)