# aplicação que mostra os conceitos de análise de regressão
# autor: Sergio Miranda Freire

library(shiny)


x <- seq(10,30,1)
colors <- colorRampPalette(c("yellow", "red3"))
        
server <- function(input, output) {

    n <- reactive ({
        input$n
    })

    z <- reactive ({
      z <- list()
      for (i in 1:n()) {
         y <- 1 + 2* x + rnorm(21, 0, 15)
         z[[i]] <- y
      }
      z <- do.call(rbind, z)
    })

    output$regressionLines <- renderPlot({
        col_range <- colors(n())
      plot(NULL, xlim=c(min(x),max(x)), ylim=c(min(z()), max(z())), ylab="Y", xlab="X")  
      for (i in 1:n()) {
           fit <- lm(z()[i,]~x)
           abline(fit$coefficients[1], fit$coefficients[2], lwd = 2, col = col_range[i])
      }  
    })    
    
    output$simulations <- renderPlot({
      par(mfrow = c(2,2))
      for (i in 1:min(4,n())) {
        plot(x, z()[i,], xlim=c(min(x),max(x)), ylim=c(min(z()), max(z())), ylab="Y", xlab="X")  
        fit <- lm(z()[i,]~x)
        abline(fit$coefficients[1], fit$coefficients[2], lwd = 2, col = "red")
        text(12,max(z())-10,paste("b1 = ", sprintf("%.1f", fit$coefficients[2])), font=2, cex=1.2)
        text(18,max(z())-10,paste("b0 = ", sprintf("%.1f", fit$coefficients[1])), font=2, cex=1.2)
      }  
      
    })    
}


ui <- fluidPage(
  
  # Application title
  titlePanel(paste(" Variabilidade dos coeficientes da reta de ", expression(regressão))),
  
  sidebarLayout(
    sidebarPanel(
        numericInput("n", 
                paste(expression(Número), " de ", expression(simulações)), 
                min = 2, 
                max = 1000, 
                value = 2
        )
  ),
  
    # Show plots
    mainPanel(
        fluidRow(
            column(8, plotOutput("regressionLines")),
          column(8, plotOutput("simulations"))
        )
    )
)
)

shinyApp(ui = ui, server = server)